/* A private interface for struct ceelog_expression.

Copyright (C) 2012 Red Hat, Inc. All rights reserved.
This copyrighted material is made available to anyone wishing to use, modify,
copy, or redistribute it subject to the terms and conditions of the GNU General
Public License v.2.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
Street, Fifth Floor, Boston, MA 02110-1301, USA.

Author: Miloslav Trmač <mitr@redhat.com> */

#ifndef EXPRESSION_PRIVATE_H
#define EXPRESSION_PRIVATE_H

#include <config.h>

#include <glib.h>
#include <pcre.h>

enum internal_op
  {
    IO_FIELD_EQ, IO_FIELD_NE, IO_FIELD_MATCHES, IO_FIELD_DOESNT_MATCH,
    IO_UNSTRUCTURED_MATCHES, IO_UNSTRUCTURED_DOESNT_MATCH,
    IO_AND, IO_OR, IO_NOT,
  };

/* A ceelog expression.  Currently expressions can only have a boolean result,
   but that might change in the future. */
/* FIXME: should support native JSON types */
struct ceelog_expr
{
  enum internal_op op;
  union
  {
    struct
    {
      char *field_path;		/* Only for IO_FIELD_*; for g_free () */
      char *value;		/* For g_free () */
    } cmp;			/* IO_FIELD_{EQ,NE}, IO_UNSTRUCTURED_* */
    struct ceelog_expr *sub[2];	/* IO_{AND,OR,NOT} */
  } v;
};

/* A compiled regex */
struct ceelog_regex
{
  pcre *re;
  pcre_extra *extra;
};

/* Compile REGEX into DEST, or set ERROR.
   Return 0 if OK, -1 on error */
G_GNUC_INTERNAL
extern int ceelog_regex_compile (struct ceelog_regex *dest, const char *regex,
				 GError **error);

/* Free RE */
G_GNUC_INTERNAL
extern void ceelog_regex_free (struct ceelog_regex *re);

#endif
